import { js_beautify, css_beautify } from 'js-beautify';
var js_beautify_config = {
  indent_size: 2,
  indent_empty_lines: true,
  e4x: true
};
var defaultDependencies = [{
  "package": 'react',
  importName: 'useState'
}, {
  "package": 'react',
  importName: 'useEffect'
}, {
  "package": 'react',
  importName: 'useRef'
}, {
  "package": 'react',
  importName: 'useLayoutEffect'
}, {
  "package": 'react',
  importName: 'useContext'
}, {
  "package": 'ahooks',
  importName: 'useRequest'
}, {
  "package": '../lowcode-common/utils',
  importName: 'utils'
}, {
  "package": '../lowcode-common/i18n',
  importName: 'i18n'
}, {
  "package": '../lowcode-common/constants',
  importName: 'constants'
}];
function getDependencies(dependencies) {
  if (dependencies === void 0) {
    dependencies = defaultDependencies;
  }
  var packageMap = new Map();

  // 1. 先按包名分组
  dependencies.forEach(function (_ref) {
    var _packageMap$get;
    var pkg = _ref["package"],
      importName = _ref.importName;
    if (!packageMap.has(pkg)) {
      packageMap.set(pkg, new Set());
    }
    (_packageMap$get = packageMap.get(pkg)) === null || _packageMap$get === void 0 ? void 0 : _packageMap$get.add(importName);
  });
  return Array.from(packageMap.entries()).map(function (_ref2) {
    var pkg = _ref2[0],
      importNames = _ref2[1];
    // 合并相同包的具名导出
    return "import { " + Array.from(importNames).join(', ') + " } from '" + pkg + "';";
  }).join('\n');
}
var initCode = function initCode(componentSchema) {
  var _initStatementCode = initStatementCode(componentSchema),
    dependencies = _initStatementCode.dependencies,
    statementCode = _initStatementCode.statementCode;
  var code = "\n  " + getDependencies(dependencies) + "\n  /** \u4E0D\u652F\u6301\u81EA\u5B9A\u4E49 import */ \n\n  export default function Page() {\n    " + statementCode + "\n    \n    // \u4E0D\u652F\u6301 return\n  }";
  return js_beautify(code, js_beautify_config);
};
export var schema2JsCode = function schema2JsCode(schema) {
  var _componentSchema$orig;
  var componentSchema = schema.componentsTree[0];
  var code = (_componentSchema$orig = componentSchema === null || componentSchema === void 0 ? void 0 : componentSchema.originCode) !== null && _componentSchema$orig !== void 0 ? _componentSchema$orig : initCode(componentSchema);

  // console.log('当前的code：', code);
  return code;
};
export var schema2CssCode = function schema2CssCode(schema) {
  var _schema$componentsTre;
  return beautifyCSS((_schema$componentsTre = schema.componentsTree[0]) === null || _schema$componentsTre === void 0 ? void 0 : _schema$componentsTre.css);
};
export var beautifyCSS = function beautifyCSS(input) {
  return input ? css_beautify(input, {
    indent_size: 2
  }) : '';
};
function initStatementCode(componentSchema) {
  if (componentSchema !== null && componentSchema !== void 0 && componentSchema.statements && componentSchema.statements.length > 0) {
    var statements = componentSchema.statements;
    var codeList = [];
    var dependencies = [].concat(defaultDependencies);
    statements.forEach(function (statement) {
      codeList.push(statement.value);
      if (statement.dependencies && statement.dependencies.length > 0) {
        dependencies.push.apply(dependencies, statement.dependencies);
      }
    });
    return {
      dependencies: dependencies,
      statementCode: codeList.join('')
    };
  } else {
    return {
      dependencies: undefined,
      statementCode: "\n    "
    };
  }
}